@@ -4,4 +4,4 @@
 # No redistribution in whole or part
 #
 def version():
-    return "1.4.3"
+    return "unpackaged"
